*The 2-colors Driver
*This driver creates a duplicate of screen objects when requested.
*One retains it's original color, and one gets it's color set to
*a user-defined color scheme.


*  Program...........: 2COLORS.PRG
*  Version...........: 1.01
*  Author............: Cory A. Jaeger
*  Created...........: 10/20/95
*  Copyright.........: (c) Wausau Coated Products, Inc.
*      Keeping with the tradition of GENSCRNX, this driver is
*     released into the public domain, to do with as you please,
*     WITH THE FOLLOWING EXCEPTIONS:
*      You must distribute 2COLORS.TXT and 2COLORS.PRG as a pair.
*      I must get credit for the work I did on this driver.
*         (In other words, don't remove my name from this text file,
*         or the comments in 2COLORS.PRG, and please don't try to
*         take credit for this effort.)
*  Description.......: This GENSCRNX driver turns a single screen
*     object into a pair of screen objects.  One of the pair will
*     retain it's original color, and the other will have it's
*     color changed to a user-defined color scheme, and any
*     custom color-pair setting will be cleared.
*        A logical expression, supplied by the screen's designer is
*     used to select the proper object at run time.
*     (*:IF directives are used.)
*
*  Enable with.......: _SCXDRV3=2COLORS  ... in config,fp
*                      *:SCXDRV3 2COLORS ... in setup snippet
*
*  Setup Directives..: *:NO2COLORS - Disable 2COLORS for this screen
*                      *:2COLORS <expL> - Default 2COLORS logical
*                         expression to be used on all screen objects.
*                         Defaults to EMPTY(<variable>).
*                          (Non-variable objects not processed in this case.)
*                         Override in object COMMENT snippets.
*                      *:2CSCHEME <expN> - Default color scheme # to
*                         use for all screen objects.
*                         Defaults to scheme 24.
*                         Override in object COMMENT snippets.
*
*  Comment Directives: *:NO2COLORS - Disable 2COLORS for single object.
*                      *:2COLORS <expL> - Logical expression to be
*                         used in *:IF directive for object.
*                      *:2CSCHEME <expN> - Color scheme # to use for
*                         this object in the "duplicate" of the object.
*
#DEFINE HeaderObj       1

IF NOT VAL(m.gsxversion) >= 2.0
   = Warning(PROGRAM() +  requires GENSCRNX v2.0.)
   GO BOTTOM
   RETURN .F.
ENDIF



*********************************************=
* Overrides:
*   Overridable with m._2COLORS="OFF",
*   DOS and Windows objects only,
*   When in DOS or Windows only.
*********************************************=
IF (Objtype = HeaderObj .AND. .NOT. drvenable(PROGRAM())).OR.;
   .NOT. INLIST(ALLTRIM(m.platform_),'DOS','WINDOWS').OR.;
   _MAC .OR. _UNIX

   GOTO BOTTOM
   RETURN .F.
ENDIF


*********************************************=
* Screen level OverRide.
*********************************************=

PRIVATE m.cDefExpr, m.cDefScheme, m.lDefExpr, m.lDefScheme

IF objtype = HeaderObj

   * IF *:NO2COLORS directive is found, don't process screen.
   IF wordsearch("*:NO2COLORS", .T., .F., .T.) > 0 ;
      GO BOTTOM
      RETURN .F.
   ENDIF

   * Check for a "MASTER" *:2COLORS directive
   m.lDefExpr = (wordsearch("*:2COLORS", .T., .F., .T.) > 0)

   * Get the "MASTER" logical expression.
   m.cDefExpr = UPPER(wordsearch("*:2COLORS", .T., .F., .F.))
   m.cDefExpr = STRTRAN(m.cDefExpr, CHR(0))

   * Check for a "MASTER" *:2CSCHEME directive
   m.lDefScheme = (wordsearch("*:2CSCHEME", .T., .F., .T.) > 0)

   * Get the "MASTER" color scheme.
   m.cDefScheme = UPPER(wordsearch("*:2CSCHEME", .T., .F., .F.))
   m.cDefScheme = STRTRAN(m.cDefScheme, CHR(0))
   IF ISDIGIT(m.cDefScheme)
      m.cDefScheme = STR(VAL(m.cDefScheme))
   ELSE
      IF m.lDefScheme
         =Warning(PROGRAM()+": Invalid Default Color Scheme.")
         m.cDefScheme = "24"
      ENDIF
   ENDIF

   * Prevent double-processing of screen
   IF m.lDefExpr
      REPLACE setupcode WITH STRTRANC(setupcode, "*:2COLORS", "*-:2COLORS")
   ENDIF
   IF m.lDefScheme
      REPLACE setupcode WITH STRTRANC(setupcode, "*:2CSCHEME", "*-:2CSCHEME")
   ENDIF
ELSE
   =Warning(PROGRAM()+": Error finding header record!")
   GO BOTTOM
   RETURN .F.
ENDIF

*********************************************=
*  Adjust "colorable" objects.
*********************************************=
PRIVATE m.cCurExpr, m.cCurScheme, m.lCurExpr, m.lCurScheme
PRIVATE m.cIfExpr, m.lIfProtect
SCAN FOR INLIST (objtype, 5, 6, 7) OR BETWEEN(objtype, 11, 16)

   IF wordsearch("*:NO2COLORS", .F., .F., .T.) > 0
      LOOP
   ENDIF

   * Check for a *:2COLORS directive
   m.lCurExpr = (wordsearch("*:2COLORS", .F., .F., .T.) > 0)

   * Get the logical expression.
   m.cCurExpr = UPPER(wordsearch("*:2COLORS", .F., .F., .F.))
   m.cCurExpr = STRTRAN(m.cCurExpr, CHR(0))

   * Check for a *:2CSCHEME directive
   m.lCurScheme = (wordsearch("*:2CSCHEME", .F., .F., .T.) > 0)

   * Get the color scheme.
   m.cCurScheme = UPPER(wordsearch("*:2CSCHEME", .F., .F., .F.))
   m.cCurScheme = STRTRAN(m.cCurScheme, CHR(0))
   IF ISDIGIT(m.cCurScheme)
      m.cCurScheme = STR(VAL(m.cCurScheme))
   ELSE
      IF m.lCurScheme
         =Warning(PROGRAM()+": Invalid Color Scheme.")
      ENDIF
      m.cCurScheme = "24"
   ENDIF

   * Prevent double-processing of object.
   IF m.lCurExpr
      REPLACE comment WITH STRTRANC(comment, "*:2COLORS", "*-:2COLORS")
   ENDIF
   IF m.lCurScheme
      REPLACE comment WITH STRTRANC(comment, "*:2CSCHEME", "*-:2CSCHEME")
   ENDIF

   IF !m.lCurExpr AND !m.lDefExpr
      LOOP
   ENDIF

   IF !m.lCurExpr
      m.cCurExpr = m.cDefExpr
   ENDIF
   IF !m.lCurScheme AND m.lDefScheme
      m.cCurScheme = m.cDefScheme
   ENDIF

   IF EMPTY(m.cCurExpr)
      IF INLIST(objtype, 11, 12, 13, 14, 15, 16)
         IF objtype = 15 AND objcode = 0
            m.cCurExpr = "EMPTY("+ALLTRIM(expr)+")"
         ELSE
            m.cCurExpr = "EMPTY("+ALLTRIM(name)+")"
         ENDIF
      ELSE
         * Don't default non-variable screen objects
         LOOP
      ENDIF
   ELSE
      m.cCurExpr = "(" + m.cCurExpr + ")"
   ENDIF

   * Get any current *:IF directives.
   m.cIfExpr = wordsearch("*:IF", .F., .F., .F.)
   m.cIfExpr = STRTRAN(m.cIfExpr, CHR(0))

   * Do we need to protect the old IF expression with parens?
   m.lIfProtect = ((LEFT(m.cIfExpr,1)+RIGHT(m.cIfExpr,1)) # "()")

   IF !EMPTY(m.cIfExpr)
      IF m.lIfProtect
         m.cIfExpr = "(" + m.cIfExpr + ") AND "
      ELSE
         m.cIfExpr = m.cIfExpr + " AND "
      ENDIF
   ENDIF

   * Duplicate the object
   =duprec(1)

   * We are at the new duplicate now.
   * Add the needed *:IF directive.
   =wordstuff("*:IF " + m.cIfExpr + m.cCurExpr, .T., .T.)

   * Set colors for duplicate.
   REPLACE scheme WITH VAL(m.cCurScheme) ;
     colorpair WITH ""

   * Go back to the original object.
   SKIP -1

   * We are back at the original now.
   * Add the needed *:IF directive.
   =wordstuff("*:IF " + m.cIfExpr + "!" + m.cCurExpr, .T., .T.)

   * Go back to duplicate so SCAN will move to next screen object.
   SKIP

ENDSCAN

RETURN .T.
